#import "AWSDKPlugin.h"
#import "AWSDKBridge.h"

static NSString * const CALLBACK_SCHEME = @"awsdkcallback";

@interface AWSDKPlugin ()

@property (strong, nonatomic) NSString *persistedCallbackId;
@property (strong, nonatomic) AWSDKBridge *bridge;

@end

@implementation AWSDKPlugin

- (void) pluginInitialize {
    self.bridge = [AWSDKBridge sharedInstance];
    self.bridge.bridgeDelegate = self;
    [self.bridge startAirwatchSDK];
}

- (void) setPersistentCallback: (CDVInvokedUrlCommand *) command {
    self.persistedCallbackId = [command callbackId];
}

- (void) allowCopyPaste: (CDVInvokedUrlCommand *) command {
    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus: CDVCommandStatus_OK
                                                        messageAsBool: [self.bridge allowCopyPaste]];

    [self.commandDelegate sendPluginResult: pluginResult
                                callbackId: [command callbackId]];
}

- (void) allowOffline: (CDVInvokedUrlCommand *) command {
    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus: CDVCommandStatus_OK
                                                        messageAsBool: [self.bridge allowOffline]];

    [self.commandDelegate sendPluginResult: pluginResult
                                callbackId: [command callbackId]];
}

- (void) customSettings: (CDVInvokedUrlCommand *) command {
    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus: CDVCommandStatus_OK
                                                      messageAsString: [self.bridge customSettings]];

    [self.commandDelegate sendPluginResult: pluginResult
                                callbackId: [command callbackId]];
}


- (void) restrictDocumentToApps: (CDVInvokedUrlCommand *) command {
    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus: CDVCommandStatus_OK
                                                        messageAsBool: [self.bridge restrictDocumentToApps]];

    [self.commandDelegate sendPluginResult: pluginResult
                                callbackId: [command callbackId]];
}

- (void) allowedApplications: (CDVInvokedUrlCommand *) command {
    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus: CDVCommandStatus_OK
                                                       messageAsArray: [self.bridge allowedApplications]];

    [self.commandDelegate sendPluginResult: pluginResult
                                callbackId: [command callbackId]];
}

- (void) groupId: (CDVInvokedUrlCommand *) command {

    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus: CDVCommandStatus_OK
                                                       messageAsString: [self.bridge groupId]];

    [self.commandDelegate sendPluginResult: pluginResult
                                callbackId: [command callbackId]];
}

- (void) username: (CDVInvokedUrlCommand *) command {

    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus: CDVCommandStatus_OK
                                                      messageAsString: [self.bridge username]];

    [self.commandDelegate sendPluginResult: pluginResult
                                callbackId: [command callbackId]];
}

- (void) serverName: (CDVInvokedUrlCommand *) command {

    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus: CDVCommandStatus_OK
                                                      messageAsString: [self.bridge serverName]];

    [self.commandDelegate sendPluginResult: pluginResult
                                callbackId: [command callbackId]];
}

- (void) isCompliant: (CDVInvokedUrlCommand *) command {

    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus: CDVCommandStatus_OK
                                                        messageAsBool: [self.bridge isCompliant]];

    [self.commandDelegate sendPluginResult: pluginResult
                                callbackId: [command callbackId]];
}

- (void) isCompromised: (CDVInvokedUrlCommand *) command {

    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus: CDVCommandStatus_OK
                                                        messageAsBool: [self.bridge isCompromised]];

    [self.commandDelegate sendPluginResult: pluginResult
                                callbackId: [command callbackId]];
}

#pragma mark AWSDKBridgeDelegateMethods

- (void) wipe {
    [self sendEvent: @"wipe"
     withDictionary: @{}];
}

- (void) lock {
    [self sendEvent: @"lock"
     withDictionary: @{}];
}

- (void) unlock {
    [self sendEvent: @"unlock"
     withDictionary: @{}];
}

- (void) stopNetworkActivity: (AWNetworkActivityStatus) networkActivityStatus {
    [self sendEvent: @"stopNetworkActivity"
     withDictionary: @{@"status" : @(networkActivityStatus)}];
}

- (void) resumeNetworkActivity {
    [self sendEvent: @"resumeNetworkActivity"
     withDictionary: @{}];
}



- (void) sendEvent: (NSString *) eventName
    withDictionary: (NSDictionary *) eventData {

    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus: CDVCommandStatus_OK
                                                  messageAsDictionary: @{@"eventName" : eventName,
                                                                         @"eventData" : eventData}];

    [pluginResult setKeepCallbackAsBool: YES]; // so Cordova keeps this callback ID alive for future use
    [self.commandDelegate sendPluginResult: pluginResult
                                callbackId: self.persistedCallbackId];
    
}


@end
